package org.example;

import com.itextpdf.text.*;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class FilteredInvoice {
    long timestamp;
    UUID uuid = UUID.randomUUID();
    String name;
    String seller;
    String buyer;
    ArrayList<Car> cars;
    float totalPrice;
    float totalPriceVAT;
    HashMap<String, String> info = new HashMap<>();
    boolean isGenerated = false;
    boolean isInfoAdded = false;

    public FilteredInvoice(ArrayList<Car> cars) {
        this.timestamp = System.currentTimeMillis();
        this.name = generateName(this.timestamp);
        this.seller = "sprzedawca";
        this.buyer = "kupujacy";
        this.cars = cars;
        calculateTotalPrice();
    }

    static String generateName(long timestamp) {
        String pattern = "yyyy/dd/MM/HH/mm/ss";
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);
        return "VAT/" + simpleDateFormat.format(timestamp);
    }

    void calculateTotalPrice() {
        this.totalPrice = 0;
        this.totalPriceVAT = 0;
        for (Car car : this.cars) {
            this.totalPrice += car.price;
            this.totalPriceVAT += car.price * (car.vat/100f);
        }
    }

    public void addInfo(String filterBy, int rangeFrom, int rangeTo) {
        isInfoAdded = true;
        info.put("filterBy", filterBy);
        if(Objects.equals(filterBy, "none")) return;
        info.put("rangeFrom", String.valueOf(rangeFrom));
        info.put("rangeTo", String.valueOf(rangeTo));
    }

    public void generate() {
        Document document = new Document();
        String path = "invoices/vat-" + this.uuid.toString() + ".pdf";
        try {
            PdfWriter.getInstance(document, new FileOutputStream(path));
        } catch (DocumentException | FileNotFoundException e) {
            throw new RuntimeException(e);
        }

        document.open();
        Font headerFont = FontFactory.getFont("Roboto Black", 20, BaseColor.BLACK);
        Font contentFont = FontFactory.getFont("Roboto", 12, BaseColor.BLACK);

        Paragraph header = new Paragraph("Faktura: " + this.name, headerFont);
        Paragraph info = getInfoParagraph(contentFont);
        PdfPTable table = getTable();

        Paragraph sum = new Paragraph(
            "Suma: " + this.totalPrice + " PLN" +
            "\nSuma VAT: " + this.totalPriceVAT + " PLN" +
            "\nSuma brutto: " + (this.totalPrice + this.totalPriceVAT) + " PLN" +
            "\n\n", contentFont);

        try {
            document.add(header);
            document.add(info);
            document.add(table);
            document.add(sum);
        } catch (DocumentException e) {
            throw new RuntimeException(e);
        }

        document.close();
        this.isGenerated = true;
    }

    private Paragraph getInfoParagraph(Font contentFont) {
        if (!isInfoAdded) return new Paragraph("", contentFont);
        String filterString = "";
        String filterBy = info.get("filterBy");
        switch (filterBy) {
            case "none" -> filterString = "Faktura za wszystkie auta";
            case "year" -> filterString = "Faktura za auta wyprodukowane w latach: " + info.get("rangeFrom") + " - " + info.get("rangeTo");
            case "price" -> filterString = "Faktura za auta w cenie: " + info.get("rangeFrom") + " - " + info.get("rangeTo");
        }

        return new Paragraph(
            "Nabywca: " + this.buyer +
            "\nSprzedawca: " + this.seller +
            "\n" + filterString +
            "\n\n", contentFont);
    }

    private PdfPTable getTable() {
        PdfPTable table = new PdfPTable(7);
        table.setWidthPercentage(100);
        table.setSpacingBefore(10f);
        table.setSpacingAfter(10f);

        String[] headers = {"l.p", "Model", "Rok", "Kolor", "Cena netto", "VAT", "Cena brutto"};
        for (String header : headers) {
            PdfPCell cell = new PdfPCell(new Phrase(header));
            cell.setHorizontalAlignment(Element.ALIGN_CENTER);
            cell.setBackgroundColor(BaseColor.LIGHT_GRAY);
            table.addCell(cell);
        }

        int i = 1;
        for (Car car : this.cars) {
            table.addCell(String.valueOf(i));
            table.addCell(car.getModel());
            table.addCell(String.valueOf(car.getYear()));
            table.addCell(car.color);
            table.addCell(String.valueOf(car.price));
            table.addCell(car.vat + "%");
            table.addCell(String.valueOf(car.price + car.price * (car.vat/100f)));
            i++;
        }

        return table;
    }
}
