package org.example;

import com.google.gson.Gson;
import spark.Request;
import spark.Response;

import javax.servlet.MultipartConfigElement;
import javax.servlet.ServletException;
import javax.servlet.http.Part;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.UUID;

import static spark.Spark.*;

public class Main {
    private static int nextID = 1;
    private static final ArrayList<Car> cars = new ArrayList<>();
    static Gson gson = new Gson();
    private static final FilteredInvoices invoices = new FilteredInvoices();

    public static void main(String[] args) {
        String projectDir = System.getProperty("user.dir");
        String staticDir = "/src/main/resources/public";
        staticFiles.externalLocation(projectDir + staticDir);

        port(8080);
        post("/add", Main::onAdd);
        get("/json", Main::onJSON);
        post("/delete", Main::onDelete);
        post("/update", Main::onUpdate);
        post("/generate", Main::onGenerate);
        post("/invoice", Main::onInvoiceAdd);
        get("/invoices/:id", Main::onInvoiceGet);
        get("/photos/:file", Main::onPhotoGet);
        post("/filteredinvoice", Main::onFilteredInvoiceAdd);
        get("/filteredinvoices", Main::onFilteredInvoicesGet);
        get("/gallery/:id", Main::onGalleryGet);
        post("/gallery", Main::onGalleryUpload);
        post("/gallerySave", Main::onGallerySave);
        get("/galleryPhotos/:file", Main::onGalleryPhotoGet);
        post("/galleryEdit", Main::onGalleryEdit);
    }

    private static String onAdd(Request req, Response res) {
        res.header("Content-Type", "application/json");
        try {
            Car car = gson.fromJson(req.body(), Car.class);
            addCar(car);

            return gson.toJson(car);
        } catch (Exception e) {
            return "{\"error\": \"Invalid car data\"}";
        }
    }

    private static void addCar(Car car) {
        car.id = nextID++;
        car.uuid = java.util.UUID.randomUUID();
        cars.add(car);
    }

    private static String onJSON(Request req, Response res) {
        res.header("Content-Type", "application/json");
        return gson.toJson(cars);
    }

    private static String onDelete(Request req, Response res) {
        res.header("Content-Type", "application/json");
        // body is {"uuid": "some-uuid"}

        int index = getCarIndexFromUUIDinJSON(req.body());
        if(index == -1) return "{\"error\": \"Car not found\"}";

        cars.remove(index);
        return "{\"message\": \"ok\"}";
    }

    private static String onUpdate(Request req, Response res) {
        res.header("Content-Type", "application/json");
        // body is {"uuid": "some-uuid", "model": "some-model", "year": 2020}

        int index = getCarIndexFromUUIDinJSON(req.body());
        if(index == -1) return "{\"error\": \"Car not found\"}";

        java.util.HashMap<String, Object> map = gson.fromJson(req.body(), java.util.HashMap.class);
        Car car = cars.get(index);
        car.setModel(map.get("model").toString());
        car.setYear(Integer.parseInt(map.get("year").toString()));
        return gson.toJson(car);
    }

    private static void addRandomCar() {
        String[] models = {"Ford", "Toyota", "Honda", "Tesla", "BMW", "Mercedes", "Audi", "Volkswagen", "Ferrari", "Lamborghini"};
        String model = models[(int) (Math.random() * models.length)];
        int intColor = (int) (Math.random() * 0xffffff);
        String color = String.format("#%06x", intColor);
        int year = ( 2023 - (int)(Math.random() * 40));
        String[] airbagDescriptions = {"driver", "passenger", "rear", "rearSide"};
        Airbag[] airbags = new Airbag[4];
        for(int i = 0; i < 4; i++) {
            airbags[i] = new Airbag(airbagDescriptions[i], Math.random() > 0.5);
        }
        CustomDate purchaseDate = CustomDate.random(2000, 20);
        float price = (float) (Math.random() * 100000);
        float[] vats = {0, 7, 22};
        float vat = vats[(int) (Math.random() * vats.length)];

        Car car = new Car(model, color, year, airbags, purchaseDate, price, vat);
        addCar(car);
    }

    private static String onGenerate(Request req, Response res) {
        res.header("Content-Type", "application/json");
        for(int i = 0; i < 10; i++) {
            addRandomCar();
        }

        return "{\"message\": \"ok\"}";
    }

    private static int getCarIndexFromUUIDinJSON (String json) {
        UUID uuid = UUID.fromString(gson.fromJson(json, java.util.HashMap.class).get("uuid").toString());
        return getCarIndexFromUUID(uuid);
    }

    private static int getCarIndexFromUUID(UUID uuid) {
        for(int i = 0; i < cars.size(); i++) {
            if(cars.get(i).uuid.equals(uuid)) return i;
        }
        return -1;
    }

    private static String onInvoiceAdd(Request req, Response res) {
        int index = getCarIndexFromUUIDinJSON(req.body());
        if(index == -1) return "{\"error\": \"Car not found\"}";
        Car car = cars.get(index);
        car.generateInvoice();

        return "{\"message\": \"ok\"}";
    }

    private static String onInvoiceGet(Request req, Response res) {
        String name = req.params(":id");
        return Helper.getAsset("invoices", name + ".pdf", res);
    }

    private static String onPhotoGet(Request req, Response res) {
        String fileName = req.params(":file");
        return Helper.getAsset("photos", fileName, res);
    }

    private static String onFilteredInvoiceAdd(Request req, Response res) {
        HashMap json = gson.fromJson(req.body(), HashMap.class);
        String filterBy = json.get("filterBy").toString();
        int rangeFrom = Integer.parseInt(json.get("rangeFrom").toString());
        int rangeTo = Integer.parseInt(json.get("rangeTo").toString());
        invoices.addInvoice(filterBy, rangeFrom, rangeTo);

        return "{\"message\": \"ok\"}";
    }

    private static String onFilteredInvoicesGet(Request req, Response res) {
        ArrayList<FilteredInvoice> allInv = invoices.getInvoices();
        return gson.toJson(allInv);
    }

    private static String onGalleryGet(Request req, Response res) {
        UUID uuid = UUID.fromString(req.params(":id"));
        int index = getCarIndexFromUUID(uuid);
        if(index == -1) return "{\"error\": \"Car not found\"}";
        Car car = cars.get(index);

        return gson.toJson(car.getGalleryPhotos());
    }

    private static String onGalleryUpload(Request req, Response res) {
        req.attribute("org.eclipse.jetty.multipartConfig", new MultipartConfigElement("/galleryPhotos"));
        ArrayList<String> uploadedPaths = new ArrayList<>();

        try {
            for(Part p : req.raw().getParts()){
                InputStream inputStream = p.getInputStream();
                byte[] bytes = inputStream.readAllBytes();
                UUID uuid = UUID.randomUUID();
                String filePath = "galleryPhotos/" + uuid + ".jpg";
                uploadedPaths.add(filePath);
                FileOutputStream fos = new FileOutputStream(filePath);
                fos.write(bytes);
                fos.close();
            }
        } catch (IOException | ServletException e) {
            throw new RuntimeException(e);
        }

        return gson.toJson(uploadedPaths);
    }

    private static String onGalleryPhotoGet(Request req, Response res) {
        String fileName = req.params(":file");
        return Helper.getAsset("galleryPhotos", fileName, res);
    }

    private static String onGallerySave(Request req, Response res) {
        HashMap json = gson.fromJson(req.body(), HashMap.class);
        ArrayList<String> paths = (ArrayList<String>) json.get("paths");

        UUID uuid = UUID.fromString(json.get("uuid").toString());
        int carIndex = getCarIndexFromUUID(uuid);
        if(carIndex == -1) return "{\"error\": \"Car not found\"}";
        Car car = cars.get(carIndex);
        car.addGalleryPhotos(paths.toArray(new String[0]));

        return "{\"message\": \"ok\"}";
    }

    private static String onGalleryEdit(Request req, Response res) {
        HashMap json = gson.fromJson(req.body(), HashMap.class);
        String filename = json.get("filename").toString();
        String effect = json.get("effect").toString();
        Object paramsObj = json.get("params");
        ArrayList<Integer> params = new ArrayList<>();

        for(Object param : (ArrayList) paramsObj) {
            int value = Math.round(Float.parseFloat(param.toString()));
            params.add(value);
        }

        String filepath = "galleryPhotos/" + filename;

        try {
            Helper.applyPhotoEffect(filepath, effect, params);
        } catch (IOException e) {
            return "{\"error\": \"Error while processing image\"}";
        }
        return "{\"message\": \"ok\"}";
    }

    public static ArrayList<Car> getCars() {
        return cars;
    }
}
